/*****************************************************************
*   main.c is a shell to run GNU assembler examples that may be found at 
*             my web-site rjaan.narod.ru
*               
* Copyright (C) 2009-2015 Andrey Y. Rjavskov(aka rjaan) <rjaan@yandex.ru> 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
*****************************************************************/
#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 
#include <errno.h>
#include <unistd.h>

#include <sys/un.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "parser.h"

#ifndef __x86_64__
#include "gasrunparts/gasrunparts.h"
#endif /*__x86_64__*/

#include "gnucrunparts/gnucrunparts.h"

static void help_message ( void );

#define EXAMPLE_DIR "/usr/local/share/gasrunparts/examples"

#define RETFAILURE(msg) {      \
        if(msg) printf(msg);   \
        retval = -EXIT_FAILURE; \
        goto terminated;        \
}

#define COMPILE_EXAMPLE(exec,sources) {                           \
   const char *cwd_p = (const char *) runpart_pwd();               \
    if( cwd_p == NULL )                                           \
      RETFAILURE("error: can\'t get current working directory!")  \
    printf( "Compile \"%s\" in dir \"%s\"\n", sources, cwd_p );      \
    if ( runpart_exec ( "gcc -g -Wsequence-point -Werror -o %s %s",exec,sources) ) \
        RETFAILURE("error: can\'t compile THIS!");                   \
    printf( "Running \"%s\" in dir \"%s\"\n", exec, cwd_p );      \
    if ( runpart_exec ( "./%s",exec) == -1 )\
        RETFAILURE("error: can\'t run THIS!");}

#define ASM_GENCODE(sources) {                                     \
    char *cwd_p = (char *) runpart_pwd();                          \
    if( cwd_p == NULL )                                            \
      RETFAILURE("error: can\'t get current working directory!")   \
    printf( "Generate assembly code \"%s\" in dir \"%s\"\n", sources, cwd_p ); \
    if ( runpart_exec ( "gcc -Wsequence-point -Werror -O2 -S -c %s  -o- | less -N",sources) ) \
        RETFAILURE("error: can\'t compile THIS!");}

int
main ( int argc, char **argv )
{
  int to_show_help;
  int to_show_version;

  int retval = EXIT_SUCCESS;

  if ( (retval=cmdline_opts_parser ( argc, argv )) < 0 ) {
     help_message();
     exit(EXIT_FAILURE);
  }

#ifdef __DEBUG__
  PRINT_VARIABLE(to_show_help);
  PRINT_VARIABLE(to_show_version); 
#endif /*__DEBUG__*/

  if ( to_show_help ) {
      goto help_label;
  }

  if ( to_show_version ) {
       VERSION_BANER();
       goto terminated;
  }

  retval = EXIT_SUCCESS;
  if ( to_run_algthm_single_ll ) {
     COMPILE_EXAMPLE("algthm_single_ll", EXAMPLE_DIR "/algthm_single_ll.c");
  }else   
    if ( to_run_gnuc_vla_use ) {
     ASM_GENCODE( EXAMPLE_DIR "/vla_use.c");     
     goto success_label;
  }else 
    if ( to_run_gnuc_vla_invalid ) {
     COMPILE_EXAMPLE("vla_invalid", EXAMPLE_DIR "/vla_invalid.c");     
    goto success_label;
  }else 
    if ( to_run_gnuc_unsafe_macros_max ) {
     COMPILE_EXAMPLE("unsafe_fmacros_max", EXAMPLE_DIR "/unsafe_fmacros_max.c");     
    goto success_label;
  }else 
  if ( to_run_gnuc_stmt_expr_max ) {
     COMPILE_EXAMPLE("stmt_expr_max", EXAMPLE_DIR "/stmt_expr_max.c");     
    goto success_label;
#ifndef __x86_64__
  }else
  if( to_run_inums_addition ) {
    printf( "Result: %u\n", inums_addition() );
    goto success_label;
  }else
    if( to_run_list_regimm ) {
       printf( "Result:  %u\n", list_regimm() );
     goto success_label;
  }else
    if( to_run_xchgw_intops ) {
      printf( "Result:  0x%04X\n", xchgw_intops() );
     goto success_label;
  }else
    if( to_run_bswap_intops ) {
      printf( "Result:  0x%08X\n", bswap_intops() );
     goto success_label;
  }else
    if( to_run_trans_negative ) {
      char neg_integer = trans_negative();
      printf( "Result:  0x%X(HEX), %d(DEC)\n", neg_integer, neg_integer  );
     goto success_label;
  }else
    if( to_run_trans_positive ) {
      unsigned char neg_integer = trans_positive();
      printf( "Result:  0x%X(HEX), %d(DEC)\n", neg_integer, neg_integer  );
#endif /*__x86_64__*/
      goto success_label;
  }else
    if( to_run_gnuc_trans_negative ) {
      signed char ret_value = gnuc_trans_negative(); 
      printf( "Input value: %d(DEC), result:  %d(DEC)\n", negative_value, ret_value );
      goto success_label;

   }else
    if( to_run_gnuc_trans_bitwisenot ) {
      signed char ret_value = gnuc_trans_bitwisenot(); 
      printf( "Input value: %d(DEC)[%02X(HEX)], result: %d(DEC)[%02X(HEX)]\n", 
                                                                bitwise_value, 
                                                                bitwise_value, 
                                                                ret_value,
                                                                ret_value );
      goto success_label;
   }else
    if( to_run_gnuc_incr_pointer ) {
      integers_t* output_list_p = gnuc_increment_pointer();       

      printf( "Input data:\n\t\t byte: %d,\n\t\t word: %d,\n\t\t quardword: %d\n", number_list_p->byte,
                                                                                   number_list_p->word,
                                                                                   number_list_p->quardword );
      printf( "Result data:\n\t\t byte: %d,\n\t\t word: %d,\n\t\t quardword: %d\n", output_list_p->byte,
                                                                                    output_list_p->word,
                                                                                    output_list_p->quardword );
      printf( "Сhanging address on: %d\n", output_list_p-number_list_p );
      goto success_label;
   }else
     if( to_run_gnuc_decr_pointer ) {
       integers_t* output_list_p = gnuc_decrement_pointer();       

       printf( "Input data:\n\t\t byte: %d,\n\t\t word: %d,\n\t\t quardword: %d\n", number_list_p->byte,
                                                                                    number_list_p->word,
                                                                                    number_list_p->quardword );
       printf( "Result data:\n\t\t byte: %d,\n\t\t word: %d,\n\t\t quardword: %d\n", output_list_p->byte,
                                                                                    output_list_p->word,
                                                                                    output_list_p->quardword );
       printf( "Сhanging address on: %d\n", output_list_p-number_list_p );
      goto success_label;
  }else{
    goto help_label;
  }

terminated:
  if( retval == -EXIT_FAILURE ) puts ("\nFailure");
  else puts ("\nSuccess");
  return retval;

help_label:
   help_message();
 
success_label:
   puts ("done.");
   return retval;
}

static void help_message ( void )
{
   printf ("%s - \
runs one of assambly functions of GAS examples\n", PACKAGE_STRING );
  printf ("Usage: %s [OPTION] <EXAMPLES>\n", PACKAGE_STRING );
  printf ("\
Options:\n\
  -h, --help                 displays this help and exit\n\
  -V, --version              outputs version information and exit\n\
Examples:\n" );
#ifndef __x86_64__
printf ("\
GNU AS\n\
   inums_addition            the addition operation with two operands\n\
   list_regimm               list some usage variants of common registers with applying immediate operands\n\
   bswap_intops              provides the byte change in 32th integers\n\
   xchgw_intops              provides the byte change in 16th integers\n\
   trans_negative            shows negative number -113 in hexadecimal and decimal notations\n\
   trans_positive            shows negative number -113 in hexadecimal and decimal notations\n");
#endif /*__x86_64__*/
printf ("\
GNU C are\n\
   gnuc_trans_negative       illustrates the usage of operator \'-\' that trans positive number 113 to \n\
                             two’s complement negation.\n\
   gnuc_trans_bitwisenot     illustrates the usage of operator \'~\' that flips all of the bits\n\
   gnuc_pointer_increment    runs the GNU C example of usage the operation increment of the pointer\n\
   gnuc_stmt_expr_max        example using the expression statement (example of building compiled and executed code)\n\
   gnuc_unsafe_macros_max    why like-function macros may be unsafe (example of building uncompiled code)\n\
   gnuc_vla_invalid          why VLA can\'t be initialized\n\
   gnuc_vla_use              use VLA in an assembly code\n\
GNU C Algorithms are\n\
   algthm_single_ll          Another of many an implementation of singly linked list\n");
 }

/*eof*/

