/* 
   File parser.c contains some functions to parse options of command line.  
 
   Copyright (C) 2015  Andrey Y. Rzhavskov  <mailto:rjaan@yandex.ru>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "parser.h"

 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 #include <time.h>

 #include <sys/types.h>
 #include <sys/param.h>

 #include <unistd.h>

 int to_show_help;
 int to_show_version;
 int to_run_inums_addition;
 int to_run_list_regimm;
 int to_run_xchgw_intops;
 int to_run_bswap_intops;
 int to_run_trans_negative;
 int to_run_trans_positive;
 int to_run_gnuc_trans_negative;
 int to_run_gnuc_trans_bitwisenot;
 int to_run_gnuc_incr_pointer; 
 int to_run_gnuc_decr_pointer;
 int to_run_gnuc_stmt_expr_max;
 int to_run_gnuc_unsafe_macros_max;
 int to_run_gnuc_vla_invalid;
 int to_run_gnuc_vla_use;
 int to_run_algthm_single_ll; 

 typedef struct jump_table_struct {

	const char *name; /* a name of long option of command-line */	
        const void *jump; /* a pointer on constant of jump-label */
	
 } jump_table_struct_t;

static int compare_jump_table_structs(const void *ch_p, const void *array_p){

   const jump_table_struct_t*  a = (const jump_table_struct_t*)ch_p;
   const jump_table_struct_t*  b = (const jump_table_struct_t*)array_p;

  return strcmp ( a->name, b->name );
}

static int globalvar_by_get_option_name ( const char buf[] )
{
 jump_table_struct_t      findme = { buf, NULL };
 jump_table_struct_t	 *found; /* a jump found labels */
 static jump_table_struct_t jump_table[] = { /* the table of jump labels */
	{ "--help",              &&case_longHelp },    /* long option --help */
	{ "--version",           &&case_longVersion },  /* long option --version */
	{ "-h",                  &&case_longHelp },    /* short option '-h'  as same long option --help */
	{ "-V",                  &&case_longVersion },  /* short option '-V' as same long option --version */
        { "inums_addition",      &&case_inumsAddition },  /* argument 'inums_addition' */
        { "list_regimm",         &&case_listRegimm },   /* argument 'list_regimm' */
        { "xchgw_intops",        &&case_xchgwIntOps }, /* argument 'xchgw_intops' */
        { "bswap_intops",        &&case_bswapIntOps }, /* argument 'bswap_intops' */
        { "trans_negative",      &&case_transNegative }, /* argument 'trans_negative' */
        { "trans_positive",      &&case_transPositive }, /* argument 'trans_positive' */
        { "gnuc_trans_negative", &&case_GNUC_transNegative }, /* argument 'gnuc_trans_negative' */
        { "gnuc_trans_bitwisenot", &&case_GNUC_transBitwiseNot }, /* argument 'gnuc_trans_bitwisenot' */
        { "gnuc_pointer_increment", &&case_GNUC_incrPointer     }, /* argument 'gnuc_pointer_increment' */        
        { "gnuc_pointer_decrement", &&case_GNUC_decrPointer     }, /* argument 'gnuc_pointer_increment' */        
        { "gnuc_stmt_expr_max"    , &&case_GNUC_stmtExprMax     }, /* argument 'gnuc_stmt_expr_max' */
        { "gnuc_unsafe_macros_max", &&case_GNUC_unsafeMacrosMax }, /* argument 'gnuc_unsafe_macros_max' */
        { "gnuc_vla_invalid", &&case_GNUC_VLAinvalid   }, /* argument 'gnuc_vla_invalid' */
        { "gnuc_vla_use",   &&case_GNUC_VLAuse         }, /* argument 'gnuc_vla_use' */
        { "algthm_single_ll",   &&case_ALGTHM_LL       }, /* argument 'algthm_single_ll' */
      
};
const int jump_table_count =  /* calculating a number of jump labels into the table   */
			   sizeof(jump_table)/sizeof(jump_table_struct_t);

/* pre-sort jump label*/
 qsort (jump_table, jump_table_count,
      		      sizeof(jump_table_struct_t), compare_jump_table_structs );
/* search jump label*/
 found = bsearch ( &findme, jump_table, jump_table_count,
      		      sizeof(jump_table_struct_t), compare_jump_table_structs );

 if(!found) {
    perror("bsearch");	
    return -1; 
 }

 goto *(found->jump);

case_ALGTHM_LL:
        to_run_algthm_single_ll++;
	goto case_Success;

case_GNUC_VLAuse:
        to_run_gnuc_vla_use++;
	goto case_Success;

case_GNUC_VLAinvalid:
        to_run_gnuc_vla_invalid++;
	goto case_Success;

case_GNUC_stmtExprMax:
        to_run_gnuc_stmt_expr_max++;
	goto case_Success;

case_GNUC_unsafeMacrosMax:
        to_run_gnuc_unsafe_macros_max++;
	goto case_Success;

 case_longHelp:
 	to_show_help++;
	goto case_Success;

 case_longVersion:
 	to_show_version++;
	goto case_Success;

case_inumsAddition:
        to_run_inums_addition++;
	goto case_Success;

case_listRegimm:
        to_run_list_regimm++;
	goto case_Success;

case_xchgwIntOps:
        to_run_xchgw_intops++;
        goto case_Success;

case_bswapIntOps:
        to_run_bswap_intops++;
        goto case_Success;

case_transNegative:
        to_run_trans_negative++;
        goto case_Success;

case_transPositive:
        to_run_trans_positive++;
        goto case_Success;

case_GNUC_incrPointer:
        to_run_gnuc_incr_pointer++;
        goto case_Success;

case_GNUC_decrPointer:
        to_run_gnuc_decr_pointer++;
        goto case_Success;


case_GNUC_transBitwiseNot:
        to_run_gnuc_trans_bitwisenot++;
        goto case_Success;

case_GNUC_transNegative:
        to_run_gnuc_trans_negative++;
        goto case_Success;

case_Success:
	return 0;
}

#define OPTION_NOTFOUND()\
 printf ( "%s: invalid option -- \'%s\'\n", *argv, *p_argv_go )

#define ARGUMENT_NOTFOUND(NAME)\
 printf ( "%s: \'%s\' option required argument\n", *argv, NAME )


/*
   cmdline_opts_parser() parses  command-line options  that were  passed    into 
			 2nd argument and initializes global variables that were
			 passed since 3rd argument.
 	
		     	 On success, this function  returns  a number of options 
		         were recognized. Otherwise, it returns negative value.
*/
int cmdline_opts_parser ( const int argc, char * const *argv ) 
 {
    va_list  ap;
    int      n_parsed_opts  	   = 0;      
    int      to_go          	   = argc-1;   
    char     * const *p_argv_go    = argv+1; 
    char     is_wait_arg    	   = 0;
    int      to_show_with_pid_old  = -1;

    to_show_help                  ^= to_show_help;
    to_show_version               ^= to_show_version;
    to_run_list_regimm            ^= to_run_list_regimm;
    to_run_inums_addition         ^= to_run_inums_addition;
    to_run_xchgw_intops           ^= to_run_xchgw_intops;
    to_run_bswap_intops           ^= to_run_bswap_intops;
    to_run_trans_negative         ^= to_run_trans_negative;
    to_run_trans_positive         ^= to_run_trans_positive;
    to_run_gnuc_trans_negative    ^= to_run_gnuc_trans_negative;
    to_run_gnuc_trans_bitwisenot  ^= to_run_gnuc_trans_bitwisenot;
    to_run_gnuc_incr_pointer      ^= to_run_gnuc_incr_pointer;
    to_run_gnuc_decr_pointer      ^= to_run_gnuc_decr_pointer;
    to_run_gnuc_stmt_expr_max     ^= to_run_gnuc_stmt_expr_max;
    to_run_gnuc_unsafe_macros_max ^= to_run_gnuc_unsafe_macros_max;
    to_run_gnuc_vla_invalid       ^= to_run_gnuc_vla_invalid;
    to_run_gnuc_vla_use           ^= to_run_gnuc_vla_use;
    to_run_algthm_single_ll       ^= to_run_algthm_single_ll;
   
    for ( ; to_go && *p_argv_go; p_argv_go++, to_go-- ) {
	if( globalvar_by_get_option_name ( *p_argv_go ) < 0 ) {
	     OPTION_NOTFOUND();
	     n_parsed_opts = -1;
	     goto out;
	 }
         n_parsed_opts++;
    }	   
    return n_parsed_opts;
out:
    return  -EXIT_FAILURE;
 }

/*eof*/

